/*
 Copyright 2011-2012 George Rhoten

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 */
package com.centralnexus.thermal;

import com.lti.civil.CaptureException;
import gnu.io.*;

import java.awt.*;
import java.io.File;
import java.io.OutputStream;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.swing.*;

/**
*
* @author George Rhoten
* @author Max Ritter
*/
public class CameraPreviewFrame extends JComponent {

	private static final long serialVersionUID = 931355892401087251L;

	private StartMenu parent;
    private OutputStream output;
    private SerialPort port;
    private int width;
    private int height;

    /** Creates new Webcam */
    public CameraPreviewFrame(StartMenu parent) {
        initComponents();
        this.parent = parent;
    }

    public void run() throws CaptureException {
        try {
            endSerial();
            CommPortIdentifier portId = CommPortIdentifier.getPortIdentifier(StartMenu.SETTINGS.getProperty(StartMenu.COM_PORT));
            port = (SerialPort) portId.open("serial talk", 115200);
            output = port.getOutputStream();
            port.setSerialPortParams(115200,
                    SerialPort.DATABITS_8,
                    SerialPort.STOPBITS_1,
                    SerialPort.PARITY_NONE);
        } catch (Exception e) {
            JOptionPane.showMessageDialog(this, Messages.getString("CameraPreviewFrame.ArduinoNotFound"), //$NON-NLS-1$
                    Messages.getString("Warning"), JOptionPane.WARNING_MESSAGE); //$NON-NLS-1$
            System.err.println("Can not open port " + StartMenu.SETTINGS.getProperty(StartMenu.COM_PORT));
            //e.printStackTrace(System.err);
        }
        wcamPanel.run(StartMenu.SETTINGS.getProperty(StartMenu.CAMERA_PORT));
        setVisible(true);
    }

    public void send(char command) {
        try {
            if (output != null) {
                output.write(command);
                output.flush();
            }
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
    }

    private void endSerial() {
        try {
            if (output != null) {
                output.close();
                output = null;
            }
            if (port != null) {
                port.close();
                port = null;
            }
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }

    }

    public void stop() {
        endSerial();
        try {
            wcamPanel.stopCapture();
        } catch (CaptureException ex) {
            ex.printStackTrace(System.err);
        }
    }

    public void close() {
	    endSerial();
	    try {
	        wcamPanel.disposeCapture();
	    } catch (CaptureException ex) {
            ex.printStackTrace(System.err);
	    }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     */
    private void initComponents() {
        GridBagLayout gridbag = new GridBagLayout();
        GridBagConstraints c = new GridBagConstraints();
        setLayout(gridbag);

		// left panel
        GridBagLayout leftGridbag = new GridBagLayout();
        GridBagConstraints leftConstraints = new GridBagConstraints();

        leftpanel.setBackground(new java.awt.Color(255, 255, 255));
        leftpanel.setBorder(javax.swing.BorderFactory.createEtchedBorder());
        leftpanel.setLayout(leftGridbag);
        
        leftConstraints.fill = GridBagConstraints.BOTH;
        Insets defaultInsets = new Insets(4, 4, 4, 4);
        leftConstraints.insets = defaultInsets;

        calibration.setFont(new Font(Font.DIALOG, 1, 18));
        calibration.setText(Messages.getString("CameraPreviewFrame.Calibration")); //$NON-NLS-1$
        leftConstraints.gridwidth = GridBagConstraints.REMAINDER; // End row
        leftpanel.add(calibration, leftConstraints);

        middleselected.setBackground(new java.awt.Color(255, 255, 255));
        buttonGroup1.add(middleselected);
        middleselected.setFont(new Font(Font.DIALOG, Font.PLAIN, 12));
        middleselected.setSelected(true);
        middleselected.setText(Messages.getString("CameraPreviewFrame.MiddlePoint")); //$NON-NLS-1$
        middleselected.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                middleselectedActionPerformed(evt);
            }
        });
        leftConstraints.gridwidth = 1; // Default
        leftpanel.add(middleselected, leftConstraints);

        middleset.setFont(new Font(Font.DIALOG, 1, 12));
        middleset.setText(Messages.getString("CameraPreviewFrame.Set")); //$NON-NLS-1$
        middleset.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                middlesetActionPerformed(evt);
            }
        });
        leftConstraints.gridwidth = GridBagConstraints.REMAINDER; // End row
        leftpanel.add(middleset, leftConstraints);

        bottomselected.setBackground(new java.awt.Color(255, 255, 255));
        buttonGroup1.add(bottomselected);
        bottomselected.setFont(new Font(Font.DIALOG, Font.PLAIN, 12));
        bottomselected.setText(Messages.getString("CameraPreviewFrame.LeftBottom")); //$NON-NLS-1$
        bottomselected.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bottomselectedActionPerformed(evt);
            }
        });
        leftConstraints.gridwidth = 1; // Reset to default.
        leftpanel.add(bottomselected, leftConstraints);

        bottomset.setFont(new Font(Font.DIALOG, 1, 12));
        bottomset.setText(Messages.getString("CameraPreviewFrame.Set")); //$NON-NLS-1$
        bottomset.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bottomsetActionPerformed(evt);
            }
        });
        leftConstraints.gridwidth = GridBagConstraints.REMAINDER; // End row
        leftpanel.add(bottomset, leftConstraints);

        middleup.setText("\u2191");
        middleup.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                middleupActionPerformed(evt);
            }
        });
        leftConstraints.gridwidth = GridBagConstraints.REMAINDER; // End row
        leftConstraints.fill = GridBagConstraints.NONE;
        leftpanel.add(middleup, leftConstraints);

        middleleft.setText("\u2190");
        middleleft.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                middleleftActionPerformed(evt);
            }
        });
        leftConstraints.gridwidth = 1; // Default
        leftConstraints.anchor = GridBagConstraints.WEST;
        leftpanel.add(middleleft, leftConstraints);

        middleright.setText("\u2192");
        middleright.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                middlerightActionPerformed(evt);
            }
        });
        leftConstraints.gridwidth = GridBagConstraints.REMAINDER; // End row
        leftConstraints.anchor = GridBagConstraints.EAST;
        leftpanel.add(middleright, leftConstraints);
        leftConstraints.anchor = GridBagConstraints.CENTER;

        middledown.setText("\u2193");
        middledown.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                middledownActionPerformed(evt);
            }
        });
        leftConstraints.gridwidth = GridBagConstraints.REMAINDER; // End row
        leftpanel.add(middledown, leftConstraints);
 
        resolutionLabel.setFont(new Font(Font.DIALOG, Font.BOLD, 18));
        resolutionLabel.setText(Messages.getString("CameraPreviewFrame.Resoluion")); //$NON-NLS-1$
        leftConstraints.gridwidth = GridBagConstraints.REMAINDER; // End row
        leftConstraints.fill = GridBagConstraints.BOTH;
        leftpanel.add(resolutionLabel, leftConstraints);

        buttonGroup2.add(res1);
        res1.setFont(new Font(Font.DIALOG, 0, 14));
        res1.setSelected(true);
        res1.setText(Messages.getString("CameraPreviewFrame.Resolution64x48")); //$NON-NLS-1$
        res1.setBackground(new Color(255, 255, 255));
        leftConstraints.gridwidth = GridBagConstraints.REMAINDER; // End row
        leftConstraints.insets = new Insets(0, defaultInsets.left, 0, defaultInsets.left);
        leftpanel.add(res1, leftConstraints);

        buttonGroup2.add(res3);
        res3.setFont(new Font(Font.DIALOG, 0, 14));
        res3.setText(Messages.getString("CameraPreviewFrame.Resolution32x24")); //$NON-NLS-1$
        res3.setBackground(new Color(255, 255, 255));
        leftConstraints.gridwidth = GridBagConstraints.REMAINDER; // End row
        leftpanel.add(res3, leftConstraints);

        start.setFont(new Font(Font.DIALOG, 1, 18));
        start.setText(Messages.getString("CameraPreviewFrame.Start")); //$NON-NLS-1$
        start.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                startActionPerformed(evt);
            }
        });
        leftConstraints.insets = defaultInsets;
        leftpanel.add(start, leftConstraints);

        add(leftpanel, c);
        // end left pane
        
        c.gridwidth = GridBagConstraints.REMAINDER; //end row

        wcamPanel.setBorder(javax.swing.BorderFactory.createEtchedBorder());

        javax.swing.GroupLayout WcamPanelLayout = new javax.swing.GroupLayout(wcamPanel);
        wcamPanel.setLayout(WcamPanelLayout);
        WcamPanelLayout.setHorizontalGroup(
            WcamPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 636, Short.MAX_VALUE)
        );
        WcamPanelLayout.setVerticalGroup(
            WcamPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGap(0, 476, Short.MAX_VALUE)
        );

        c.gridwidth = GridBagConstraints.REMAINDER; // End row
        c.gridheight = 2;
        add(wcamPanel, c);

        //pack();
    }// </editor-fold>//GEN-END:initComponents

    private void middlerightActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_middlerightActionPerformed
        if (middleselected.isSelected()) {
            send('c');
        }
        else if (bottomselected.isSelected()) {
            send('h');
        }
    }//GEN-LAST:event_middlerightActionPerformed

    private void middlesetActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_middlesetActionPerformed
        if (middleselected.isSelected()) {
            send('f');
        }
    }//GEN-LAST:event_middlesetActionPerformed

    private void bottomsetActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bottomsetActionPerformed
        if (bottomselected.isSelected()) {
            send('k');
        }
    }//GEN-LAST:event_bottomsetActionPerformed

    private void middledownActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_middledownActionPerformed
        if (middleselected.isSelected()) {
            send('e');
        }
        else if (bottomselected.isSelected()) {
            send('j');
        }
    }//GEN-LAST:event_middledownActionPerformed

    private void middleleftActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_middleleftActionPerformed
        if (middleselected.isSelected()) {
            send('b');
        }
        else if (bottomselected.isSelected()) {
            send('g');
        }
    }//GEN-LAST:event_middleleftActionPerformed

    private void middleupActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_middleupActionPerformed
        if (middleselected.isSelected()) {
            send('d');
        }
        else if (bottomselected.isSelected()) {
            send('i');
        }
    }//GEN-LAST:event_middleupActionPerformed

    private void startActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_startActionPerformed
        //setVisible(false);
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd-HHmm");
        String date = sdf.format(new Date());
        String dirName = StartMenu.SETTINGS.getProperty(StartMenu.WORKING_DIRECTORY) + parent.name + " (" + date + ")";
        File f = new File(dirName);
        if (!f.isDirectory()) {
            f.mkdir();
        }
        try {
            wcamPanel.saveImage(dirName);
        } catch (CaptureException ex) {
            ex.printStackTrace(System.err);
        }

        if (res1.isSelected()) {
            width = 64;
            height = 48;
        }
        else if (res3.isSelected()) {
            width = 32;
            height = 24;
        }
        endSerial();
        ProgressBar progressbar = new ProgressBar(parent);
        progressbar.run(dirName, StartMenu.SETTINGS.getProperty(StartMenu.COM_PORT), parent.name, width, height);
        //dispose();
    }//GEN-LAST:event_startActionPerformed

    private void bottomselectedActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bottomselectedActionPerformed
        send('m');
    }//GEN-LAST:event_bottomselectedActionPerformed

    private void middleselectedActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_middleselectedActionPerformed
        send('l');
    }//GEN-LAST:event_middleselectedActionPerformed

    private JPanel leftpanel = new JPanel();
    private JLabel calibration = new JLabel();
    private JRadioButton middleselected = new JRadioButton();
    private JRadioButton bottomselected = new JRadioButton();
    private JButton bottomset = new JButton();
    private JButton middleset = new JButton();
    private ButtonGroup buttonGroup1 = new ButtonGroup();
    private ButtonGroup buttonGroup2 = new ButtonGroup();
    private JButton middleup = new JButton();
    private JButton middleleft = new JButton();
    private JButton middleright = new JButton();
    private JButton middledown = new JButton();
    private JLabel resolutionLabel = new JLabel();
    private JRadioButton res1 = new JRadioButton();
    private JRadioButton res3 = new JRadioButton();
    private JButton start = new JButton();
    private WebcamPanel wcamPanel = new WebcamPanel();


    public String getCameraID() {
        return wcamPanel.getDeviceID();
    }

    public void changeCamera(String deviceID) {
        try {
            wcamPanel.disposeCapture();
            wcamPanel.run(deviceID);
        }
        catch (CaptureException e) {
            e.printStackTrace(System.err);
        }
    }

}
