/*
 Copyright 2011-2012 George Rhoten

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 */
package com.centralnexus.thermal;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.SystemColor;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.RenderedImage;
import java.awt.image.WritableRaster;
import java.io.File;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.Formatter;
import java.util.Hashtable;
import java.util.Locale;
import java.util.Scanner;
import javax.imageio.ImageIO;
import javax.swing.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.filechooser.FileFilter;

import com.centralnexus.Utility;

/**
 *
 * @author George Rhoten
 * @author Max Ritter
 */
public class Output extends javax.swing.JPanel {

	private static final long serialVersionUID = 6966468502962301998L;

	private static final short SETTINGS_XPADDING = 32;
	private static final short DEFAULT_THERMAL_WIDTH = 640;
    private static final short DEFAULT_THERMAL_HEIGHT = 480;

    private TemperatureStatisticTracker tempTracker = new TemperatureStatisticTracker();
    private boolean mirror = false;
    private float[][] temperatures;
    private boolean[][] showtemp;
    private float max;
    private float average;
    private float min;
    private float tmin;
    private float tmax;
    private float step;
    private int rows;
    private int lines;
    private int broadness;
    private float roomtemp;
    private String name;
    private String dirName;

    /** Creates new form OutputNew */
    public Output(Window parent) {
    	super();
        initComponents();
        String mirrorStr = (String)StartMenu.SETTINGS.get(StartMenu.MIRROR_IMAGE);
        mirror = (mirrorStr == null ? false : mirrorStr.equals("true"));
    }

    public void run(String dirName, int rows, int lines) {
        this.dirName = dirName;
        this.rows = rows;
        this.lines = lines;
        this.broadness = (int) Math.round(DEFAULT_THERMAL_WIDTH / rows);
        temperatures = new float[rows][lines];
        showtemp = new boolean[rows][lines];
        TemperatureData();
        LimitValues();
        thermalimage.setIcon(convertRenderedImage(CreateThermalImage()));
        BufferedImage wcamimage;
        try {
            wcamimage = ImageIO.read(new File(dirName + "/Webcam.jpg")); //$NON-NLS-1$
        } catch (Exception ex) {
            System.out.println("Exception : " + ex.getMessage()); //$NON-NLS-1$
            wcamimage = new BufferedImage(DEFAULT_THERMAL_WIDTH, DEFAULT_THERMAL_HEIGHT, BufferedImage.TYPE_INT_RGB);
        }
        webcamimage.setIcon(convertRenderedImage(wcamimage));
        colourscale.setIcon(new ImageIcon(convertRenderedImage(createColorScale())));
        resolution.setText(rows + " x " + lines); //$NON-NLS-1$
        mouseovertemp.setText(Messages.getString("Output.NA")); //$NON-NLS-1$
        formatTemperatureStatistics();
    }

    private void initComponents() {
        MouseTracker mt = new MouseTracker();
        ActionListener scaleTracker = new ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                formatTemperatureStatistics();
            }
        };

        GridBagConstraints mainConstraints = new GridBagConstraints();
        GridBagConstraints settingsConstraints = new GridBagConstraints();

        //setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        //setTitle("Image Results");
        setBackground(SystemColor.window);
        //setResizable(false);
        setLayout(new GridBagLayout());

        thermalimage.addMouseMotionListener(mt);
        thermalimage.addMouseListener(mt);
        thermalimage.setBackground(Color.WHITE);
        thermalimage.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        thermalimage.setPreferredSize(new Dimension(DEFAULT_THERMAL_WIDTH, DEFAULT_THERMAL_HEIGHT));
        //thermalimage.setMinimumSize(new java.awt.Dimension(DEFAULT_THERMAL_WIDTH, DEFAULT_THERMAL_HEIGHT));

        mainConstraints.insets = new Insets(4, 4, 4, 4);
        mainConstraints.fill = GridBagConstraints.BOTH;
        mainConstraints.weightx = 0.1;
        mainConstraints.weighty = 0.1;
        add(thermalimage, mainConstraints);

        webcamimage.setBackground(Color.WHITE);
        webcamimage.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        webcamimage.setPreferredSize(new Dimension(DEFAULT_THERMAL_WIDTH, DEFAULT_THERMAL_HEIGHT));
        //webcamimage.setMinimumSize(new java.awt.Dimension(DEFAULT_THERMAL_WIDTH, DEFAULT_THERMAL_HEIGHT));

        mainConstraints.gridwidth = GridBagConstraints.REMAINDER; // End row
        add(webcamimage, mainConstraints);


        settingsPanel.setBackground(Color.WHITE);
        settingsPanel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
        settingsPanel.setLayout(new GridBagLayout());

        Insets defaultInsets = settingsConstraints.insets;
        settingsConstraints.ipadx = SETTINGS_XPADDING;

        minsliderlabel.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        minsliderlabel.setText(Messages.getString("Output.MinTemp")); //$NON-NLS-1$
        settingsConstraints.insets = new Insets(8, 0, 0, 0);
        settingsConstraints.anchor = GridBagConstraints.EAST;
        settingsPanel.add(minsliderlabel, settingsConstraints);

        minslider.setPaintLabels(true);
        minslider.addChangeListener(tempTracker);
        minslider.setBackground(Color.WHITE);
        settingsConstraints.anchor = GridBagConstraints.CENTER;
        settingsPanel.add(minslider, settingsConstraints);

//        settingsConstraints.gridheight = 2;
        settingsConstraints.gridwidth = 2;
        settingsPanel.add(colourscale, settingsConstraints);
//        settingsConstraints.gridheight = 1; // Reset to the default
        settingsConstraints.gridwidth = 1; // Reset to the default

        averagetemplabel.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        averagetemplabel.setText(Messages.getString("Output.AverageTemp")); //$NON-NLS-1$
        settingsConstraints.anchor = GridBagConstraints.EAST;
        settingsPanel.add(averagetemplabel, settingsConstraints);

        averagetemp.setFont(new Font(Font.DIALOG, Font.PLAIN, 14));
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsConstraints.ipadx = 0;
        settingsPanel.add(averagetemp, settingsConstraints);
        settingsConstraints.ipadx = SETTINGS_XPADDING;

        mouseovertemplabel.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        mouseovertemplabel.setText(Messages.getString("Output.HoverTemp")); //$NON-NLS-1$
        settingsConstraints.anchor = GridBagConstraints.EAST;
        settingsPanel.add(mouseovertemplabel, settingsConstraints);

        mouseovertemp.setFont(new Font(Font.DIALOG, Font.PLAIN, 14));
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsConstraints.ipadx = 0;
        settingsConstraints.gridwidth = GridBagConstraints.REMAINDER; // End row
        settingsPanel.add(mouseovertemp, settingsConstraints);
        settingsConstraints.ipadx = SETTINGS_XPADDING;
        settingsConstraints.gridwidth = 1; // Reset to the default

        maxsliderlabel.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        maxsliderlabel.setText(Messages.getString("Output.MaxTemp")); //$NON-NLS-1$
        settingsConstraints.insets = defaultInsets;
        settingsConstraints.anchor = GridBagConstraints.EAST;
        settingsPanel.add(maxsliderlabel, settingsConstraints);

        maxslider.setBackground(Color.WHITE);
        maxslider.setPaintLabels(true);
        maxslider.addChangeListener(tempTracker);
        settingsPanel.add(maxslider, settingsConstraints);

        defaultscale.setBackground(Color.WHITE);
        scaleTypeGroup.add(defaultscale);
        defaultscale.setFont(new Font(Font.DIALOG, Font.BOLD, 14)); // NOI18N
        defaultscale.setSelected(true);
        defaultscale.setText(Messages.getString("Output.BlueGreenRed")); //$NON-NLS-1$
        defaultscale.addActionListener(tempTracker);
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsPanel.add(defaultscale, settingsConstraints);

        alternativescale.setBackground(Color.WHITE);
        scaleTypeGroup.add(alternativescale);
        alternativescale.setFont(new Font(Font.DIALOG, Font.BOLD, 14)); // NOI18N
        alternativescale.setText(Messages.getString("Output.RedYellow")); //$NON-NLS-1$
        alternativescale.addActionListener(tempTracker);
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsPanel.add(alternativescale, settingsConstraints);

        maxtemplabel.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        maxtemplabel.setText(Messages.getString("Output.MaxTemp")); //$NON-NLS-1$
        settingsConstraints.anchor = GridBagConstraints.EAST;
        settingsPanel.add(maxtemplabel, settingsConstraints);

        maxtemp.setFont(new Font(Font.DIALOG, Font.PLAIN, 14));
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsPanel.add(maxtemp, settingsConstraints);

        ambienttemplabel.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        ambienttemplabel.setText(Messages.getString("Output.AmbientTemp")); //$NON-NLS-1$
        settingsConstraints.anchor = GridBagConstraints.EAST;
        settingsPanel.add(ambienttemplabel, settingsConstraints);

        ambienttemp.setFont(new Font(Font.DIALOG, Font.PLAIN, 14));
        settingsConstraints.gridwidth = GridBagConstraints.REMAINDER; // End row
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsPanel.add(ambienttemp, settingsConstraints);
        settingsConstraints.gridwidth = 1; // Reset to the default

        save.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        save.setText(Messages.getString("Output.Save")); //$NON-NLS-1$
        save.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
            	SaveIMG();
            }
        });
        settingsConstraints.insets = new Insets(0, 0, 8, 0);
        settingsConstraints.anchor = GridBagConstraints.CENTER;
        settingsPanel.add(save, settingsConstraints);

        complete.setBackground(Color.WHITE);
        complete.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        complete.setText(Messages.getString("Output.CombinedImage")); //$NON-NLS-1$
        settingsPanel.add(complete, settingsConstraints);

        celsiusButton.setBackground(Color.WHITE);
        temperatureTypeGroup.add(celsiusButton);
        celsiusButton.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        celsiusButton.setSelected(Utility.isCelsiusDefault(null));
        celsiusButton.setText(Messages.getString("Output.Celsius")); //$NON-NLS-1$
        celsiusButton.addActionListener(scaleTracker);
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsPanel.add(celsiusButton, settingsConstraints);

        fahrenheitButton.setBackground(Color.WHITE);
        temperatureTypeGroup.add(fahrenheitButton);
        fahrenheitButton.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        fahrenheitButton.setSelected(!Utility.isCelsiusDefault(null));
        fahrenheitButton.setText(Messages.getString("Output.Fahrenheit")); //$NON-NLS-1$
        fahrenheitButton.addActionListener(scaleTracker);
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsPanel.add(fahrenheitButton, settingsConstraints);

        mintemplabel.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        mintemplabel.setText(Messages.getString("Output.MinTemp")); //$NON-NLS-1$
        settingsConstraints.anchor = GridBagConstraints.EAST;
        settingsPanel.add(mintemplabel, settingsConstraints);

        mintemp.setFont(new Font(Font.DIALOG, Font.PLAIN, 14));
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsPanel.add(mintemp, settingsConstraints);

        resolutionlabel.setFont(new Font(Font.DIALOG, Font.BOLD, 14));
        resolutionlabel.setText(Messages.getString("Output.Resolution")); //$NON-NLS-1$
        settingsConstraints.anchor = GridBagConstraints.EAST;
        settingsPanel.add(resolutionlabel, settingsConstraints);

        resolution.setFont(new Font(Font.DIALOG, Font.PLAIN, 14));
        settingsConstraints.anchor = GridBagConstraints.WEST;
        settingsPanel.add(resolution, settingsConstraints);

        mainConstraints.gridwidth = 2;
        mainConstraints.fill = GridBagConstraints.HORIZONTAL;
        mainConstraints.weightx = 1.0;
        mainConstraints.weighty = 1.0;
        add(settingsPanel, mainConstraints);
        
        validate();
    }

    private JPanel settingsPanel = new JPanel();
    private JLabel ambienttemp = new JLabel();
    private JLabel ambienttemplabel = new JLabel();
    private JLabel averagetemp = new JLabel();
    private JLabel averagetemplabel = new JLabel();
    private JRadioButton defaultscale = new JRadioButton();
    private JRadioButton alternativescale = new JRadioButton();
    private ButtonGroup scaleTypeGroup = new ButtonGroup();
    private JRadioButton celsiusButton = new JRadioButton();
    private JRadioButton fahrenheitButton = new JRadioButton();
    private ButtonGroup temperatureTypeGroup = new ButtonGroup();
    private JLabel colourscale = new JLabel();
    private JCheckBox complete = new JCheckBox();
    private JSlider maxslider = new JSlider();
    private JLabel maxsliderlabel = new JLabel();
    private JLabel maxtemp = new JLabel();
    private JLabel maxtemplabel = new JLabel();
    private JSlider minslider = new JSlider();
    private JLabel minsliderlabel = new JLabel();
    private JLabel mintemp = new JLabel();
    private JLabel mintemplabel = new JLabel();
    private JLabel mouseovertemp = new JLabel();
    private JLabel mouseovertemplabel = new JLabel();
    private JLabel resolution = new JLabel();
    private JLabel resolutionlabel = new JLabel();
    private JButton save = new JButton();
    private AutoResizeImage thermalimage = new AutoResizeImage();
    private AutoResizeImage webcamimage = new AutoResizeImage();

    public void calculateMouseTemp(Point pos) {
        int xpos = (int) pos.getX();
        int ypos = (int) pos.getY();
        int xTempNumber = xpos / broadness;
        int yTempNumber = ypos / broadness;
        if (xTempNumber < rows && yTempNumber < lines && xTempNumber >= 0 && yTempNumber >= 0) {
            mouseovertemp.setText(formatTemperature(temperatures[xTempNumber][yTempNumber], true));
        } else {
            mouseovertemp.setText(Messages.getString("Output.NA")); //$NON-NLS-1$
        }
        repaint();
    }

    public void showTemp(Point pos) {
        int xpos = (int) pos.getX();
        int ypos = (int) pos.getY();
        int xTempNumber = xpos / broadness;
        int yTempNumber = ypos / broadness;
        if (xTempNumber < rows && yTempNumber < lines && xTempNumber >= 0 && yTempNumber >= 0) {
            showtemp[xTempNumber][yTempNumber] = !showtemp[xTempNumber][yTempNumber];
        }
        calculateSlider();
        repaint();
    }

    public void LimitValues() {
        max = 0.0f;
        min = temperatures[0][0];
        average = 0.0f;
        for (int x = 0; x < rows; x++) {
            for (int y = 0; y < lines; y++) {
                if (temperatures[x][y] > max) {
                    max = temperatures[x][y];
                }
                if (temperatures[x][y] < min) {
                    min = temperatures[x][y];
                }
                average += temperatures[x][y];
            }
        }
        average = average / (rows * lines);
        tmax = max;
        tmin = min;
        step = (max - min) / 10.0f;
    }

    public void SaveIMG() {
        BufferedImage img1 = convertRenderedImage(CreateThermalImage());
        img1 = img1.getSubimage(0, 0, img1.getWidth(), img1.getHeight());
        img1 = img1.getSubimage(0, 0, img1.getWidth(), img1.getHeight());

        BufferedImage img2 = convertRenderedImage(createColorScale());

        BufferedImage combined = null;
        Graphics g = null;
        if (complete.isSelected()) {
            BufferedImage wcamimage = new BufferedImage(DEFAULT_THERMAL_WIDTH, DEFAULT_THERMAL_HEIGHT, BufferedImage.TYPE_INT_RGB);
            try {
                wcamimage = ImageIO.read(new File(dirName + "/Webcam.jpg")); //$NON-NLS-1$
            }
            catch (Exception ex) {
                System.out.println("Exception : " + ex.getMessage()); //$NON-NLS-1$
            }
            combined = new BufferedImage(img1.getWidth() + wcamimage.getWidth() + 30, img1.getHeight() + 80,
                    BufferedImage.TYPE_INT_RGB);
            g = combined.getGraphics();
            g.setColor(Color.WHITE);
            g.fillRect(0, 0, img1.getWidth() + wcamimage.getWidth() + 30, img1.getHeight() + 80);
            g.drawImage(wcamimage, img1.getWidth() + 20, 10, null);
        }
        else {
            combined = new BufferedImage(img1.getWidth() + 20, img1.getHeight() + 80, BufferedImage.TYPE_INT_RGB);
            g = combined.getGraphics();
            g.setColor(Color.WHITE);
            g.fillRect(0, 0, img1.getWidth() + 20, img1.getHeight() + 80);
        }

        int xadd = (int) Math.round(((img1.getWidth() - 550) * 0.5) + 0.5);

        Font font = new Font("Times New Roman", Font.BOLD, 15); //$NON-NLS-1$
        g.setFont(font);
        SimpleDateFormat sdf = new SimpleDateFormat();
        g.setColor(Color.darkGray);
        g.drawString(name, 218 + xadd, img1.getHeight() + 30);
        g.drawImage(img1, 10, 10, null);
        g.drawImage(img2, xadd - 25, img1.getHeight() + 20, null);
        g.setFont(new Font("Times New Roman", Font.PLAIN, 14)); //$NON-NLS-1$
        g.drawString(sdf.format(new Date()), 480 + xadd, img1.getHeight() + 30);
        g.drawString(Messages.getString("Output.RoomTempAbbr") + formatTemperature(roomtemp, true), 218 + xadd, img1.getHeight() + 55); //$NON-NLS-1$
        g.drawString(Messages.getString("Output.MinTempAbbr") + formatTemperature(tmin, true), 350 + xadd, img1.getHeight() + 55); //$NON-NLS-1$
        g.drawString(Messages.getString("Output.MaxTempAbbr") + formatTemperature(tmax, true), 480 + xadd, img1.getHeight() + 55); //$NON-NLS-1$
        g.dispose();

        try {
            JFileChooser fc = new JFileChooser();
            fc.setCurrentDirectory(new File(dirName));
            fc.removeChoosableFileFilter(fc.getAcceptAllFileFilter());
            fc.setFileFilter(new JPGfilter());
            fc.setSelectedFile(new File("ThermalImage.jpg")); //$NON-NLS-1$
            int rVal = fc.showDialog(null, Messages.getString("Output.Save")); //$NON-NLS-1$
            if (rVal == JFileChooser.APPROVE_OPTION) {
                File file = fc.getSelectedFile();
                ImageIO.write(combined, "jpg", file); //$NON-NLS-1$
            }
        } catch (IOException e) {
        	e.printStackTrace(System.err);
        }
    }

    private static Color[] DEFAULT_COLOR_SCHEME = new Color[] {
        new Color(28, 1, 108),
        new Color(31, 17, 218),
        new Color(50, 111, 238),
        new Color(63, 196, 229),
        new Color(64, 222, 135),
        new Color(192, 240, 14),
        new Color(223, 172, 18),
        new Color(209, 111, 14),
        new Color(210, 50, 28),
        new Color(194, 26, 0),
        new Color(132, 26, 0)
    };
    private static Color[] ALTERNATE_COLOR_SCHEME = new Color[] {
        new Color(0, 0, 5),
        new Color(7, 1, 97),
        new Color(51, 1, 194),
        new Color(110, 2, 212),
        new Color(158, 6, 150),
        new Color(197, 30, 58),
        new Color(218, 66, 0),
        new Color(237, 137, 0),
        new Color(246, 199, 23),
        new Color(251, 248, 117),
        new Color(252, 254, 253)
    };

    private static short calculateRGB(int rgb1, int rgb2, double t1, double step, double t) {
        return (short)(rgb1 + (((t - t1) / step) * (rgb2 - rgb1)));
    }

    private Color calculateColor(Color[] colorScheme, double temperature) {
        Color val = colorScheme[0];
        if (temperature < min) {
            val = colorScheme[0];
        }
        else if (temperature >= max) {
            val = colorScheme[colorScheme.length - 1];
        }
        else {
            int step1 = (int)((temperature - min) / step);
            int step2 = step1 + 1;
            Color col1 = colorScheme[step1];
            Color col2 = colorScheme[step2];
            int red = calculateRGB(col1.getRed(), col2.getRed(), (min + step1 * step), step, temperature);
            int green = calculateRGB(col1.getGreen(), col2.getGreen(), (min + step1 * step), step, temperature);
            int blue = calculateRGB(col1.getBlue(), col2.getBlue(), (min + step1 * step), step, temperature);
            val = new Color(red, green, blue);
        }
        return val;
    }

    public RenderedImage CreateThermalImage() {
        int xWidthFactor = broadness;
        int yWidthFactor = broadness;
        BufferedImage bufferedImage = new BufferedImage(DEFAULT_THERMAL_WIDTH, DEFAULT_THERMAL_HEIGHT, BufferedImage.TYPE_INT_RGB);
        Graphics2D g2d = bufferedImage.createGraphics();

        g2d.setColor(Color.WHITE);
        g2d.fillRect(0, 0, DEFAULT_THERMAL_WIDTH, DEFAULT_THERMAL_HEIGHT);
        Color[] colorScheme = defaultscale.isSelected() ? DEFAULT_COLOR_SCHEME : ALTERNATE_COLOR_SCHEME;
        ArrayList<Float> perimeterVals = new ArrayList<Float>(4);
        for (int x = 0; x < rows; x++) {
            for (int y = 0; y < lines; y++) {
                g2d.setColor(calculateColor(colorScheme, temperatures[x][y]));
                g2d.fillRect((int)(x * xWidthFactor + xWidthFactor * 0.25), (int)(y * yWidthFactor + yWidthFactor * 0.25), xWidthFactor / 2, yWidthFactor / 2);
                if (y + 1 < lines) {
                    g2d.setColor(calculateColor(colorScheme, (temperatures[x][y] + temperatures[x][y + 1]) / 2));
                    g2d.fillRect((int)(x * xWidthFactor + xWidthFactor * 0.25), (int)(y * yWidthFactor + yWidthFactor * 0.75), xWidthFactor / 2, yWidthFactor / 2);
                }
                if (x + 1 < rows) {
                    g2d.setColor(calculateColor(colorScheme, (temperatures[x][y] + temperatures[x + 1][y]) / 2));
                    g2d.fillRect((int)(x * xWidthFactor + xWidthFactor * 0.75), (int)(y * yWidthFactor + yWidthFactor * 0.25), xWidthFactor / 2, yWidthFactor / 2);
                    if (y + 1 < lines) {
                        // Calculate the median (not the mean). This provides better contour smoothing.
                        perimeterVals.add(temperatures[x][y]);
                        perimeterVals.add(temperatures[x][y + 1]);
                        perimeterVals.add(temperatures[x + 1][y]);
                        perimeterVals.add(temperatures[x + 1][y + 1]);
                        Collections.sort(perimeterVals);
                        g2d.setColor(calculateColor(colorScheme, (perimeterVals.get(1) + perimeterVals.get(2)) / 2));
                        g2d.fillRect((int)(x * xWidthFactor + xWidthFactor * 0.75), (int)(y * yWidthFactor + yWidthFactor * 0.75), xWidthFactor / 2, yWidthFactor / 2);
                    }
                }
                perimeterVals.clear();
            }
        }
        // Draw text on top of color background.
        g2d.addRenderingHints(new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON));
        g2d.setFont(new Font("Arial", Font.BOLD, 15)); //$NON-NLS-1$
        for (int x = 0; x < rows; x++) {
            for (int y = 0; y < lines; y++) {
                if (showtemp[x][y]) {
                    int ix = (int) (x * xWidthFactor);
                    int iy = (int) (y * yWidthFactor);
                    g2d.setColor(Color.black);
                    String tempStr = formatTemperature(temperatures[x][y], true);
                    g2d.fillOval(ix + (int)(xWidthFactor * 0.25), iy + (int)(yWidthFactor * 0.25), (int) Math.round(xWidthFactor * 0.5), (int) Math.round(yWidthFactor * 0.5));
                    // Make sure we don't clip the point's text.
                    Rectangle2D textBounds = g2d.getFontMetrics().getStringBounds(tempStr, g2d);
                    int strOffsetX = ix - (int)(textBounds.getWidth() / 2 - broadness / 2);
                    int strOffsetY = iy + (int)textBounds.getHeight() + broadness / 2;
                    if (strOffsetX < 0) {
                        strOffsetX = 0;
                    }
                    if (strOffsetX + textBounds.getWidth() > thermalimage.getWidth()) {
                        strOffsetX = thermalimage.getWidth() - (int)textBounds.getWidth();
                    }
                    if (strOffsetY + textBounds.getHeight() > thermalimage.getHeight()) {
                        strOffsetY = iy;
                    }
                    g2d.drawString(tempStr, strOffsetX, strOffsetY);
                }
            }
        }
        g2d.dispose();

        return bufferedImage;
    }

    public RenderedImage createColorScale() {
        BufferedImage bufferedImage = new BufferedImage(230, 32, BufferedImage.TYPE_INT_RGB);
        try {
            if (defaultscale.isSelected()) {
                bufferedImage = ImageIO.read(this.getClass().getResource("Colorscale1.png")); //$NON-NLS-1$
            } else if (alternativescale.isSelected()) {
                bufferedImage = ImageIO.read(this.getClass().getResource("Colorscale2.png")); //$NON-NLS-1$
            }
        } catch (Exception ex) {
            System.out.println("Exception : " + ex.getMessage()); //$NON-NLS-1$
        }
        Graphics2D g2d = bufferedImage.createGraphics();
        g2d.addRenderingHints(new RenderingHints(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON));
        g2d.setFont(new Font("Arial", Font.BOLD, 12)); //$NON-NLS-1$
        g2d.setColor(Color.black);
        g2d.drawString(formatTemperature(min, false), 3, 30);
        g2d.drawString(formatTemperature(min + step, false), 26, 30);
        g2d.drawString(formatTemperature(min + 2 * step, false), 49, 30);
        g2d.drawString(formatTemperature(min + 3 * step, false), 72, 30);
        g2d.drawString(formatTemperature(min + 4 * step, false), 95, 30);
        g2d.drawString(formatTemperature(min + 5 * step, false), 118, 30);
        g2d.drawString(formatTemperature(min + 6 * step, false), 141, 30);
        g2d.drawString(formatTemperature(min + 7 * step, false), 164, 30);
        g2d.drawString(formatTemperature(min + 8 * step, false), 187, 30);
        g2d.drawString(formatTemperature(max, false), 212, 30);
        g2d.dispose();

        return bufferedImage;
    }

    public void TemperatureData() {
        try {
            Scanner scanner = new Scanner(new File(dirName + "/tempvalues.txt")); //$NON-NLS-1$
            scanner.useLocale(Locale.US);
            int i = 0;
            int values = rows * lines;
            if (mirror) {
                for (int x = 0; x < rows; x++) {
                    for (int y = lines - 1; y >= 0; y--) {
                        if ((scanner.hasNextFloat()) && (i <= values)) {
                            temperatures[x][y] = scanner.nextFloat();
                            i++;
                        }
                    }
                }
            }
            else {
                for (int x = rows - 1; x >= 0; x--) {
                    for (int y = lines - 1; y >= 0; y--) {
                        if ((scanner.hasNextFloat()) && (i <= values)) {
                            temperatures[x][y] = scanner.nextFloat();
                            i++;
                        }
                    }
                }
            }
            roomtemp = scanner.nextFloat();
            name = scanner.next();
        } catch (Exception e) {
        	e.printStackTrace(System.err);
            JOptionPane.showMessageDialog(null, Messages.getString("WebcamPanel.ErrorReadingTemperatureFile"), //$NON-NLS-1$ 
                    Messages.getString("Error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$ 
        	return;
        }
    }

    public BufferedImage convertRenderedImage(RenderedImage img) {
        if (img instanceof BufferedImage) {
            return (BufferedImage) img;
        }
        ColorModel cm = img.getColorModel();
        int width = img.getWidth();
        int height = img.getHeight();
        WritableRaster raster = cm.createCompatibleWritableRaster(width, height);
        boolean isAlphaPremultiplied = cm.isAlphaPremultiplied();
        Hashtable<String, Object> properties = new Hashtable<String, Object>();
        String[] keys = img.getPropertyNames();
        if (keys != null) {
            for (int i = 0; i < keys.length; i++) {
                properties.put(keys[i], img.getProperty(keys[i]));
            }
        }
        BufferedImage result = new BufferedImage(cm, raster, isAlphaPremultiplied, properties);
        img.copyData(raster);
        return result;
    }

	private void formatTemperatureStatistics() {
	    tempTracker.setEnabled(false);
		ambienttemp.setText(formatTemperature(roomtemp, true));
        averagetemp.setText(formatTemperature(average, true));
        mintemp.setText(formatTemperature(tmin, true));
        maxtemp.setText(formatTemperature(tmax, true));

        float middletemp;
        if ((roomtemp > tmin) && (roomtemp < tmax)) {
            middletemp = roomtemp;
        } else {
            middletemp = Math.round(tmin + ((tmax - tmin) * 0.5));
        }
        int maxVal = (int) Math.round(convertToCurrentTemperatureType(tmax + 25));
        int minVal = (int) Math.round(convertToCurrentTemperatureType(middletemp + 1));
        int val = (int) Math.round(convertToCurrentTemperatureType(tmax));
        int bigspace = (int) ((maxVal - minVal) * 0.20);
        int smallspace = (int) (0.5 * bigspace);
        maxslider.setMinimum(val - smallspace);
        maxslider.setMaximum(val + bigspace * 4);
        maxslider.setValue(val);
        maxslider.setMinorTickSpacing(smallspace);
        maxslider.setMajorTickSpacing(bigspace);

        maxVal = (int) Math.round(convertToCurrentTemperatureType(middletemp - 1));
        minVal = (int) Math.round(convertToCurrentTemperatureType(tmin - 25));
        val = (int) Math.round(convertToCurrentTemperatureType(tmin));
        minslider.setMinimum(minVal);
        minslider.setMaximum(maxVal);
        minslider.setValue(val);
        bigspace = (int) ((maxVal - minVal) * 0.20);
        smallspace = (int) (0.5 * bigspace);
        minslider.setMinimum(val - bigspace * 4);
        minslider.setMaximum(val + smallspace);
        minslider.setValue(val);
        minslider.setMinorTickSpacing(smallspace);
        minslider.setMajorTickSpacing(bigspace);

        // Now that the values have settled down. Show the new values.
        tempTracker.setEnabled(true);
        calculateSlider();
	}

    public void calculateSlider() {
        max = convertFromCurrentTemperatureType(maxslider.getValue());
        min = convertFromCurrentTemperatureType(minslider.getValue());
        step = (max - min) / 10.0f;
        colourscale.setIcon(new ImageIcon(convertRenderedImage(createColorScale())));
        thermalimage.setIcon(convertRenderedImage(CreateThermalImage()));
        thermalimage.repaint();
    }

    private class TemperatureStatisticTracker implements ChangeListener, ActionListener {
        boolean enableTracking = true;

        @Override
        public synchronized void stateChanged(ChangeEvent arg0) {
            if (enableTracking) {
                calculateSlider();
            }
        }

        @Override
        public synchronized void actionPerformed(ActionEvent e) {
            if (enableTracking) {
                calculateSlider();
            }
        }
        
        public synchronized void setEnabled(boolean val) {
            enableTracking = val;
        }
    }

    private String formatTemperature(float val, boolean showLongFormat) {
        StringBuilder sb = new StringBuilder();
        Formatter formatter = new Formatter(sb);
        String format;
        if (showLongFormat) {
            format = "%1$.2f\u00B0%2$s"; //$NON-NLS-1$
        }
        else {
            format = "%1$.0f\u00B0"; //$NON-NLS-1$
        }

        if (celsiusButton.isSelected()) {
            formatter.format(format, val, (showLongFormat ? Messages.getString("Output.CelsiusAbbr") : "")); //$NON-NLS-1$ //$NON-NLS-2$
        }
        else {
            formatter.format(format, convertToFahrenheit(val), (showLongFormat ? Messages.getString("Output.FahrenheitAbbr") : "")); //$NON-NLS-1$ //$NON-NLS-2$
        }
        return sb.toString();
    }

    private static float convertToFahrenheit(float val) {
        return val * 9 / 5 + 32;
    }
    
    private static float convertFromFahrenheit(float val) {
        return (val - 32) * 5 / 9;
    }

    /**
     * 
     * @param val Celsius
     * @return Celsius or Fahrenheit
     */
    private float convertToCurrentTemperatureType(float val) {
        if (celsiusButton.isSelected()) {
            return val;
        }
        else {
            return convertToFahrenheit(val);
        }
    }

    /**
     * 
     * @param val Celsius or Fahrenheit
     * @return Celsius
     */
    private float convertFromCurrentTemperatureType(float val) {
        if (celsiusButton.isSelected()) {
            return val;
        }
        else {
            return convertFromFahrenheit(val);
        }
    }

	private class MouseTracker implements MouseListener, MouseMotionListener {
        @Override
        public void mouseDragged(MouseEvent e) {
        }

        @Override
        public void mouseMoved(MouseEvent e) {
            calculateMouseTemp(e.getPoint());
        }

        @Override
        public void mouseClicked(MouseEvent e) {
            if (e.getButton() == MouseEvent.BUTTON1) {
                showTemp(e.getPoint());
            }
        }

        @Override
        public void mouseExited(MouseEvent e) {
            mouseovertemp.setText(Messages.getString("Output.NA")); //$NON-NLS-1$
        }

        @Override
        public void mousePressed(MouseEvent e) {
        }

        @Override
        public void mouseReleased(MouseEvent e) {
        }

        @Override
        public void mouseEntered(MouseEvent e) {
        }
    }

    public class JPGfilter extends FileFilter {
        @Override
        public boolean accept(File f) {
            return (f.getName().toLowerCase().endsWith(".jpg") //$NON-NLS-1$
                    || f.getName().toLowerCase().endsWith(".jpeg")); //$NON-NLS-1$
        }

        @Override
        public String getDescription() {
            return Messages.getString("Output.JpegTypeInfo"); //$NON-NLS-1$
        }
    }
}
