/*
 Copyright 2011-2012 George Rhoten

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 */
package com.centralnexus.thermal;

import gnu.io.CommPortIdentifier;
import gnu.io.NoSuchPortException;
import gnu.io.PortInUseException;
import gnu.io.SerialPort;
import gnu.io.SerialPortEvent;
import gnu.io.SerialPortEventListener;
import gnu.io.UnsupportedCommOperationException;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Toolkit;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import java.util.regex.Pattern;

import javax.swing.JDialog;
import javax.swing.JOptionPane;
import javax.swing.JPanel;

/**
 *
 * @author George Rhoten
 * @author Max Ritter
 */
public class ProgressBar extends JDialog implements SerialPortEventListener {

	private static final long serialVersionUID = 357793947402536140L;
	private String dirName;
    private String name;
    private String comport;
    private float[] tempvalues;
    private String[] strinput = new String[1];
    private int roomtemp = 0;
    private boolean[] checktemp;
    private int values;
    private int count;
    private int rows;
    private int lines;
    private long startTime;
    private SerialPort port;
    private File out;
    private PrintWriter pw;
    private InputStream input;
    private OutputStream output;
    private StartMenu parent;

    /** Creates new form ProgressBarNew */
    public ProgressBar(StartMenu parent) {
    	super(parent);
    	this.parent = parent;
        initComponents();
    }

    public void run(String dirName, String comport, String name, int rows, int lines) {
        this.dirName = dirName;
        this.comport = comport;
        this.name = name;
        this.rows = rows;
        this.lines = lines;
        File f = new File(dirName);
        if (f.isDirectory()) {
        } else {
            f.mkdir();
        }
        values = rows * lines;
        strinput = new String[(int) Math.round(values * 1.2)];
        current.setMaximum((int) Math.round(values + 3.0));
        tempvalues = new float[values];
        checktemp = new boolean[values];
        updateTimeStatus();
        startTime = System.currentTimeMillis();
        try {
            CommPortIdentifier portId = CommPortIdentifier.getPortIdentifier(comport);
            port = (SerialPort) portId.open("serial talk", 115200); //$NON-NLS-1$
            input = port.getInputStream();
            output = port.getOutputStream();
            port.setSerialPortParams(115200,
                    SerialPort.DATABITS_8,
                    SerialPort.STOPBITS_1,
                    SerialPort.PARITY_NONE);
            port.addEventListener(this);
            port.notifyOnDataAvailable(true);
        } catch (Exception e) {
            e.printStackTrace(System.err);
            JOptionPane.showMessageDialog(parent, Messages.getString("ProgressBar.NotConnected"), //$NON-NLS-1$
                    Messages.getString("Error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
            return;
        }
        try {
            Thread.sleep(1500);
            if (values == 3072) {
                send('n');
            }
            if (values == 1728) {
                send('o');
            }
            if (values == 768) {
                send('p');
            }
            if (values == 192) {
                send('q');
            }
            output.write('a');
            output.flush();
            setVisible(true);
        } catch (Exception e) {
        	e.printStackTrace(System.err);
        }
    }

    public void send(char command) throws IOException, InterruptedException {
        output.write(command);
        output.flush();
    }

    public synchronized void close() throws IOException, InterruptedException {
        if (output != null) {
            send('~'); // Reset
            output.close();
            output = null;
        }
        if (input != null) {
            input.close();
            input = null;
        }
        if (port != null) {
            port.close();
            port = null;
        }
    }

    @Override
    public synchronized void serialEvent(SerialPortEvent oEvent) {
        if (oEvent.getEventType() == SerialPortEvent.DATA_AVAILABLE) {
            try {
            	do {
	                char inChar;
	                StringBuilder strbuilder = new StringBuilder();
	                while ((inChar = (char)input.read()) != '\n') {
	                	strbuilder.append(inChar);
	                }
	                String incoming = strbuilder.toString().trim();
	                if (incoming.charAt(0) == 'r') {
	                    roomtemp = Integer.parseInt(incoming.substring(1));
	                } else if (incoming.startsWith("Stop") == true) { //$NON-NLS-1$
	                    finish();
	                } else {
	                    strinput[count] = incoming;
	                    count++;
	                    current.setValue(count);
	                    updateTimeStatus();
	                }
            	}
	            while (input.available() > 0);
            } catch (Exception e) {
            	e.printStackTrace(System.err);
            }
        }
    }
    
    private static final double PIXELS_PER_SECOND = 7.45;

    private void updateTimeStatus() {
        int time = (int) Math.round((values - count) / PIXELS_PER_SECOND);
        if (time >= 0) {
            status.setText((time / 60) + ":" + ((time % 60) < 10 ? "0" : "") + (time % 60));
        }
    }

    public void finish() {
        stringhandling();
        try {
            out = new File(dirName + "/tempvalues.txt"); //$NON-NLS-1$
            pw = new PrintWriter(new FileWriter(out.getPath(), true));
            //pw.println(0);
            pw.flush();
            for (int i = 0; i < values; i++) {
                if (checktemp[i] == false) {
                    if (i > 0) {
                        tempvalues[i] = tempvalues[i - 1];
                        System.err.println(i);
                    } else {
                        tempvalues[i] = 0;
                        System.err.println(i);
                    }
                }
                pw.println(tempvalues[i]);
            }
            pw.println(roomtemp);
            pw.println(name);
            pw.println("r" + rows); //$NON-NLS-1$
            pw.println("l" + lines); //$NON-NLS-1$
            pw.flush();
            pw.close();
            setVisible(false);
            parent.displayProject(dirName, rows, lines);
            long endTime = System.currentTimeMillis();
            System.out.println("Elapsed time=" + ((endTime - startTime) / 1000) + " seconds");
        } catch (Exception e) {
        	e.printStackTrace(System.err);
        }
    }

    public void stringhandling() {
        float temp = 0.0f;
        int number = 0;
        Pattern splitPattern = Pattern.compile("\t"); //$NON-NLS-1$
        String errorStr = "";
        
        for (int i = 0; i < count; i++) {
            if (strinput[i] != null) {
                try {
                    String[] strVals = splitPattern.split(strinput[i]);
                    number = Integer.parseInt(strVals[0]);
                    temp = Float.parseFloat(strVals[1]);
                    if ((number <= values) && (number >= 1)) {
                        tempvalues[number - 1] = temp;
                        checktemp[number - 1] = true;
                    }
                }
                catch (Exception e) {
                    errorStr += strinput[i] + '\n';
                    System.out.println("Failed to parse at index " + i + ": " + strinput[i]);
                    e.printStackTrace(System.err);
                }
            }
        }
        if (errorStr.length() > 0) {
            JOptionPane.showMessageDialog(parent, Messages.getString("ProgressBar.ParseError") + '\n' + errorStr, //$NON-NLS-1$
                    Messages.getString("Error"), JOptionPane.ERROR_MESSAGE); //$NON-NLS-1$
        }
    }

    private void initComponents() {

        current = new javax.swing.JProgressBar();
        remainingtime = new javax.swing.JLabel();
        status = new javax.swing.JLabel();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle(Messages.getString("ProgressBar.Title")); //$NON-NLS-1$
        setResizable(false);
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                formWindowClosing(evt);
            }
        });
        getContentPane().setLayout(new BorderLayout());

        current.setBackground(new java.awt.Color(255, 255, 255));
        current.setForeground(new java.awt.Color(51, 204, 255));
        current.setMaximumSize(new java.awt.Dimension(220, 29));
        current.setMinimumSize(new java.awt.Dimension(220, 29));
        current.setPreferredSize(new java.awt.Dimension(220, 29));
        current.setStringPainted(true);
        getContentPane().add(current, BorderLayout.CENTER);

        JPanel remainingPanel = new JPanel();
        remainingtime.setFont(new java.awt.Font(Font.SANS_SERIF, 1, 14));
        remainingtime.setText(Messages.getString("ProgressBar.RemainingTime")); //$NON-NLS-1$
        remainingPanel.add(remainingtime);

        status.setFont(new java.awt.Font(Font.SANS_SERIF, 0, 14));
        remainingPanel.add(status);
        add(remainingPanel, BorderLayout.SOUTH);
        
        pack();
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        setBounds((screenSize.width-256)/2, (screenSize.height-116)/2, getSize().width, getSize().height);
    }

	private void formWindowClosing(java.awt.event.WindowEvent evt) {
	    try {
	        close();
	        // Reset the port
	        try {
	            CommPortIdentifier portId = CommPortIdentifier.getPortIdentifier(comport);
                port = (SerialPort) portId.open("serial talk", 115200); //$NON-NLS-1$
                port.setSerialPortParams(115200,
                        SerialPort.DATABITS_8,
                        SerialPort.STOPBITS_1,
                        SerialPort.PARITY_NONE);
	        } catch (NoSuchPortException ex) {
	            ex.printStackTrace(System.err);
	        } catch (PortInUseException ex) {
                ex.printStackTrace(System.err);
	        } catch (UnsupportedCommOperationException ex) {
                ex.printStackTrace(System.err);
            }
	        Thread.sleep(1000);
	        close();
	    } catch (IOException ex) {
            ex.printStackTrace(System.err);
	    } catch (InterruptedException ex) {
            ex.printStackTrace(System.err);
	    }
	}

    private javax.swing.JProgressBar current;
    private javax.swing.JLabel remainingtime;
    private javax.swing.JLabel status;
}
